require File.expand_path(File.dirname(__FILE__) + '/spec_helper')
##
# This spec uses college courses and their prerequisites as
# the content to test the MultirootTree structure
##

	##
	# Danny Pham
	# 12/10/12
	##

describe MultirootTree do
	it "adds a child" do
		e0 = MultirootTree.new("ENGL 073")
		e1 = MultirootTree.new("ENGL 092")

		e0.add_child(e1)

		e0.children.should include(e1)
	end

	it "adds a child then the child adds its own child" do
		f0 = MultirootTree.new("FRCH 121")
		f1 = MultirootTree.new("FRCH 122")
		f2 = MultirootTree.new("FRCH 123")

		f0.add_child(f1)
		f1.add_child(f2)

		(f0.children.should include(f1)) && (f1.children.should include(f2))
	end

	it "adds a child and the child has it as a parent" do
		f0 = MultirootTree.new("FRCH 121")
		f1 = MultirootTree.new("FRCH 122")

		f0.add_child(f1)

		f1.parents.should include(f0)
	end

	it "adds line of descendants" do 
		p0 = MultirootTree.new("PROG 110")
		p1 = MultirootTree.new("PROG 118")
		p2 = MultirootTree.new("PROG 120")
		p3 = MultirootTree.new("PROG 140")
		p4 = MultirootTree.new("PROG 210")
		p5 = MultirootTree.new("PROG 225")

		p0.add_descendant_line(p1, p2, p3, p4, p5)

		((p0.children.should include(p1)) && \
		(p1.children.should include(p2)) && \
		(p2.children.should include(p3)) && \
		(p3.children.should include(p4)) && \
		(p4.children.should include(p5))) == true
	end

	it "adds a parent" do
		d1 = MultirootTree.new("BUSIT 103")
		d2 = MultirootTree.new("DBA 130")

		d2.add_parent(d1)

		d2.parents.should include(d1)
	end

	it "adds multiple parents" do
		p3 = MultirootTree.new("PROG 140")
		
		m3 = MultirootTree.new("MATH 099")
		e1 = MultirootTree.new("ENGL 101")
		d2 = MultirootTree.new("DBA 130")

		p3.add_parents(m3, e1, d2)
		p3.parents.should include(m3, e1, d2)
	end

	it "adds a parent and the parent adds it as a child" do
		d1 = MultirootTree.new("BUSIT 103")
		d2 = MultirootTree.new("DBA 130")

		d2.add_parent(d1)	
		
		d1.children.should include(d2)	
	end

	it "adds children then returns itself when finding roots (greatest parents)" do
		m0 = MultirootTree.new("MATH 075")
		m1 = MultirootTree.new("MATH 097")
		m2 = MultirootTree.new("MATH 098")
		m3 = MultirootTree.new("MATH 099")

		m0.add_child(m1)
		m1.add_child(m2)
		m2.add_child(m3)

		(m3.roots.inspect == m0.inspect && m0.roots.inspect == m0.inspect)
	end

	it "adds parents then returns the last parent as the root" do
		m0 = MultirootTree.new("MATH 075")
		m1 = MultirootTree.new("MATH 097")
		m2 = MultirootTree.new("MATH 098")
		m3 = MultirootTree.new("MATH 099")

		m3.add_parent(m2)
		m2.add_parent(m1)
		m1.add_parent(m0)

		(m3.roots.inspect == m0.inspect && m0.roots.inspect == m0.inspect)
	end

	it "finds floor (youngest children)" do
		m0 = MultirootTree.new("MATH 075")
		m1 = MultirootTree.new("MATH 097")
		m2 = MultirootTree.new("MATH 098")
		m3 = MultirootTree.new("MATH 099")

		m3.add_parent(m2)
		m2.add_parent(m1)
		m1.add_parent(m0)

		m0.floor.inspect == m3.inspect
	end

	it "disowns a child" do
		m0 = MultirootTree.new("MATH 075")
		m1 = MultirootTree.new("MATH 097")

		m0.add_child(m1)

		m0.disowns(m1)

		(m1.parents.size == 0 && m0.children.size == 0)
	end

	it "emancipates from a parent" do
		m0 = MultirootTree.new("MATH 075")
		m1 = MultirootTree.new("MATH 097")

		m0.add_child(m1)

		m1.emancipates_from(m0)

		(m1.parents.size == 0 && m0.children.size == 0)		
	end
end



